@@ -55,6 +55,7 @@ song_change_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/song_change")
 xchat_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/xchat")
 email_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/email")
 gaim_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/gaim")
+pidgin_prefs = gconfsync.GConfSync("/apps/gnome-osd/plugins/pidgin")
 listener = None # Bonobo/ObjectDirectory:activation:register listener
 screensaver_active = False
 
@@ -352,6 +353,76 @@ def rhythmbox_dbus_song_change_cb(uri):
     osd.send("<message id='rhythmbox' hide_timeout='10000'>" + song + "</message>")
 
 
+## --------- pidgin
+def format_ppidgin(buddy, action):
+    return "<span foreground='yellow'>%s</span> " \
+           "<span size='smaller' style='italic'>%s</span>" % (buddy, action)
+
+def pidgin_event_buddyon_cb(buddyid):
+    global pidgin_prefs, osd
+    if not pidgin_prefs['enabled']:
+        return
+    bus = dbus.SessionBus()
+    pidginobj = bus.get_object("im.pidgin.purple.PurpleService", "/im/pidgin/purple/PurpleObject")
+    pidgin = dbus.Interface(pidginobj, "im.pidgin.purple.PurpleInterface")
+    alias = pidgin.PurpleBuddyGetAlias(buddyid)
+    message = format_ppidgin(alias, "logs in")
+    osd.send("<message id='pidgin' hide_timeout='3000'>" + message + "</message>")
+
+def pidgin_event_buddyoff_cb(buddyid):
+    global pidgin_prefs, osd
+    if not pidgin_prefs['enabled']:
+        return
+    bus = dbus.SessionBus()
+    pidginobj = bus.get_object("im.pidgin.purple.PurpleService", "/im/pidgin/purple/PurpleObject")
+    pidgin = dbus.Interface(pidginobj, "im.pidgin.purple.PurpleInterface")
+    alias = pidgin.PurpleBuddyGetAlias(buddyid)
+    message = format_ppidgin(alias, "logs out")
+    osd.send("<message id='pidgin' hide_timeout='3000'>" + message + "</message>")
+
+def pidgin_event_message_cb(account, name):
+    global pidgin_prefs, osd
+    if not pidgin_prefs['enabled']:
+        return
+    bus = dbus.SessionBus()
+    pidginobj = bus.get_object("im.pidgin.purple.PurpleService", "/im/pidgin/purple/PurpleObject")
+    pidgin = dbus.Interface(pidginobj, "im.pidgin.purple.PurpleInterface")
+    buddy = pidgin.PurpleFindBuddy(account, name)
+    if buddy != 0:
+        alias = pidgin.PurpleBuddyGetAlias(buddy)
+    else:
+        alias = name
+    message = format_ppidgin(alias, "is writing a message")
+    osd.send("<message id='pidgin' hide_timeout='3000'>" + message + "</message>")
+
+def pidgin_event_messagereceived_cb(account, name, message, conversation, flags):
+    global pidgin_prefs, osd
+    if not pidgin_prefs['enabled']:
+        return
+    bus = dbus.SessionBus()
+    pidginobj = bus.get_object("im.pidgin.purple.PurpleService", "/im/pidgin/purple/PurpleObject")
+    pidgin = dbus.Interface(pidginobj, "im.pidgin.purple.PurpleInterface")
+    buddy = pidgin.PurpleFindBuddy(account, name)
+    if buddy != 0:
+        alias = pidgin.PurpleBuddyGetAlias(buddy)
+    else:
+        alias = name
+    message = format_ppidgin(alias, "sent a message")
+    osd.send("<message id='pidgin' hide_timeout='3000'>" + message + "</message>")
+
+def monitor_pidgin(bus):
+    bus.add_signal_receiver(pidgin_event_buddyon_cb, "BuddySignedOn",
+                            "im.pidgin.purple.PurpleInterface")
+    bus.add_signal_receiver(pidgin_event_buddyoff_cb, "BuddySignedOff",
+                            "im.pidgin.purple.PurpleInterface")
+    bus.add_signal_receiver(pidgin_event_message_cb, "BuddyTyping",
+                            "im.pidgin.purple.PurpleInterface")
+    bus.add_signal_receiver(pidgin_event_messagereceived_cb, "ReceivedImMsg",
+                            "im.pidgin.purple.PurpleInterface")
+## --------- /pidgin
+
+## --------- gaim
+
 def format_pgaim(buddy, action):
     return "<span foreground='yellow'>%s</span> " \
            "<span size='smaller' style='italic'>%s</span>" % (buddy, action)
@@ -417,7 +488,7 @@ def monitor_gaim(bus):
                             "net.sf.gaim.GaimInterface")
     bus.add_signal_receiver(gaim_event_messagereceived_cb, "ReceivedImMsg",
                             "net.sf.gaim.GaimInterface")
-
+## --------- /gaim
 
 def monitor_dbus_apps():
     bus = dbus.SessionBus()
@@ -431,7 +502,7 @@ def monitor_dbus_apps():
                             "org.freedesktop.DBus", arg0=XCHAT_DBUS_SERVICE)
     bus.add_signal_receiver(screensaver_active_cb, "ActiveChanged",
                             "org.gnome.ScreenSaver")
-    monitor_gaim(bus)
+    monitor_pidgin(bus)
     try:
         monitor_xchat(bus)
     except dbus.DBusException, ex:
